@@ -4,7 +4,7 @@
 (*  Copyright 2012,2015 OCamlPro                                          *)
 (*                                                                        *)
 (*  All rights reserved.This file is distributed under the terms of the   *)
-(*  GNU Lesser General Public License version 3.0 with linking            *)
+(*  GNU Lesser General Public License version 2.1 with linking            *)
 (*  exception.                                                            *)
 (*                                                                        *)
 (*  TypeRex is distributed in the hope that it will be useful,            *)
@@ -283,8 +283,6 @@ let rec close_top_let = function
 let rec unwind f path = match path with
   | { kind } :: _ when f kind -> path
   | { kind=KCodeInComment } :: _ -> path
-  | { kind = KAttrId _ } :: { kind } :: _ when f kind -> path
-  (* never remove the KattrId following a KExtendedItem *)
   | _ :: path -> unwind f path
   | [] -> []
 
@@ -301,7 +299,7 @@ let unwind_while f path =
   | _ -> None
 
 let top_kind = function
-  | KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _ -> true
+  | KStruct|KSig|KParen|KBegin|KObject|KExtendedItem _|KAttrId _ -> true
   | _ -> false
 
 let stritem_kind = function
@@ -604,7 +602,7 @@ let rec update_path config block stream tok =
                     let indent = h.column + pad in
                     if indent < 0 then None
                     else Some ({ h with kind; indent; column=indent;
-                                        line_indent = indent;
+                                        line_indent = indent-pad;
                                         pad = -pad } :: p)
         in
         match negative_indent () with
@@ -763,10 +761,15 @@ let rec update_path config block stream tok =
   | SEMISEMI    -> append KUnknown L ~pad:0 (unwind_top block.path)
   | INCLUDE     -> append KInclude L (unwind_top block.path)
   | EXCEPTION   ->
-      let p = unwind (function KExpr _ -> false | _ -> true) block.path in
-      (match p with
-       | {kind=KWith KMatch|KBar KMatch}::_ -> append expr_atom L block.path
-       | _ -> append KException L (unwind_top block.path))
+      (match last_token block with
+       | Some LET ->
+           append KUnknown L block.path (* let exception *)
+       | _ ->
+           let p = unwind (function KExpr _ -> false | _ -> true) block.path in
+           (match p with
+            | {kind=KWith KMatch|KBar KMatch}::_ ->
+                append expr_atom L block.path
+            | _ -> append KException L (unwind_top block.path)))
   | BEGIN       -> open_paren KBegin block.path
   | OBJECT      -> append KObject L block.path
   | VAL         -> append KVal L (unwind_top block.path)
@@ -1115,7 +1118,8 @@ let rec update_path config block stream tok =
             | KWith(KMatch|KTry) | KBar(KMatch|KTry) | KArrow(KMatch|KTry)
             | KFun
             | KBody(KType|KExternal) | KColon
-            | KStruct | KSig | KObject -> true
+            | KStruct | KSig | KObject
+            | KExtendedItem _ | KExtendedExpr _ -> true
             | _ -> false)
             path
         in
@@ -1172,13 +1176,14 @@ let rec update_path config block stream tok =
         | KExternal | KModule | KType | KLet | KLetIn | KException | KVal
         | KBar KType
         | KStruct | KSig | KObject
-        | KAnd(KModule|KType|KLet|KLetIn) -> true
+        | KAnd(KModule|KType|KLet|KLetIn)
+        | KExtendedItem _ | KExtendedExpr _ -> true
         | _ -> false
       in
       let rec find_parent path =
         let path = unwind unwind_to path in
         (match path with
-         | [] | {kind=KCodeInComment}::_ ->
+         | [] | {kind=KCodeInComment|KExtendedItem _|KExtendedExpr _}::_ ->
              make_infix tok block.path
          | {kind=KBody KType}::p -> (* type t = t' = ... *)
              (match p with
@@ -1286,6 +1291,9 @@ let rec update_path config block stream tok =
        | Some ({kind=KExpr _} :: {kind=KType} :: ({kind=KColon} :: _ as p)) ->
            (* let f: type t. t -> t = ... *)
            p
+       | Some ({kind=KExpr 200} :: ({kind=KColon} :: {kind=KLet} :: _ as p)) ->
+           (* method m : 'x 'y . ... =   (KLet is actually "method") *)
+           p
        | Some ({kind=KExpr i} :: ({kind=KBrace|KWith KBrace} as h :: p))
          when (i = prio_max || i = prio_dot) && next_offset tok stream = None ->
            (* special case: distributive { Module. field; field } *)
@@ -1368,6 +1376,8 @@ let rec update_path config block stream tok =
              ) (atom block.path)
        | _ -> atom block.path)
 
+  | LIDENT s when String.length s > 0 && s.[0] = '\'' ->
+      append (KExpr prio_max) L ~pad:0 block.path
   | INT64 _ | INT32 _ | INT _ | LIDENT _
   | FLOAT _ | CHAR _ | STRING _
   | TRUE | FALSE | NATIVEINT _
@@ -1437,14 +1447,28 @@ let rec update_path config block stream tok =
               let col = Path.indent p + Path.pad p in
               append (KComment (tok, col)) (A col) ~pad block.path
             in
+            (* if we are directly after a case in a sum-type, use that for
+               alignment *)
+            let align_bar =
+              if tok.newlines > 1 || not (is_inside_type block0.path)
+              then None
+              else
+                let find_bar =
+                  unwind_while
+                    (function KBar _ | KExpr _ -> true | _ -> false)
+                    block0.path
+                in match find_bar with
+                | Some ({kind=KBar _; column}::_) -> Some column
+                | _ -> None
+            in
             (* after a closed expr: look-ahead *)
-            (match next_token_full stream with
-             | None -> blocklevel ()
+            (match next_token_full stream, align_bar with
+             | None, None -> blocklevel ()
              | Some ((* full block-closing tokens + newline *)
                  {token = SEMISEMI | DONE | END
                           | GREATERRBRACE | GREATERRBRACKET | RBRACE
                           | RBRACKET | RPAREN }
-               , _)
+               , _), _
                when tok.newlines > 1 ->
                  blocklevel ()
              | Some ((* semi block-closing tokens *)
@@ -1452,37 +1476,21 @@ let rec update_path config block stream tok =
                           | GREATERRBRACE | GREATERRBRACKET | RBRACE
                           | RBRACKET | RPAREN
                           | THEN | ELSE | IN | EQUAL }
-               , _)
+               , _), None
                when tok.newlines <= 1 -> (* indent as above *)
                  let col = (Path.top block0.path).line_indent in
                  append (KComment (tok, col)) (A col) ~pad block.path
-             | next ->
-                 (* indent like next token, _unless_ we are directly after a
-                    case in a sum-type *)
-                 let align_bar =
-                   if tok.newlines > 1 || not (is_inside_type block0.path)
-                   then None
-                   else
-                     let find_bar =
-                       unwind_while
-                         (function KBar _ | KExpr _ -> true | _ -> false)
-                         block0.path
-                     in match find_bar with
-                     | Some ({kind=KBar _; column}::_) -> Some column
-                     | _ -> None
+             | _, Some indent ->
+                 append (KComment (tok,indent)) (A indent) ~pad block.path
+             | next, None -> (* recursive call to indent like next line *)
+                 let path = match next with
+                   | Some ({token = EOF }, _) | None -> []
+                   | Some (next,stream) ->
+                       update_path config block stream
+                         { next with newlines = tok.newlines }
                  in
-                 match align_bar with
-                 | Some indent ->
-                     append (KComment (tok,indent)) (A indent) ~pad block.path
-                 | None -> (* recursive call to indent like next line *)
-                     let path = match next with
-                       | Some ({token = EOF }, _) | None -> []
-                       | Some (next,stream) ->
-                           update_path config block stream
-                             { next with newlines = tok.newlines }
-                     in
-                     let col = Path.indent path in
-                     append (KComment (tok,col)) (A col) ~pad block.path)
+                 let col = Path.indent path in
+                 append (KComment (tok,col)) (A col) ~pad block.path)
         | _ ->
             let col = Path.indent block.path + Path.pad block.path in
             append (KComment (tok,col)) (A col) ~pad block.path)
@@ -1558,7 +1566,7 @@ let reverse t =
               :: r
           | n1::n2::p ->
               { n1 with indent = col; column = col }
-              :: { n2 with pad = n2.pad + diff }
+              :: { n2 with indent = n2.indent + diff }
               :: p
           | [] -> []
         in
