@@ -8,9 +8,9 @@ let should_check_can_sell_and_marking regulatory_regime =
   | `foo
     -> some_function
          argument
-(* pszilagyi: The above typically occurs in a multi-pattern match clause, so the
-   clause expression is on a line by itself.  This is the more typical way a
-   long single-pattern match clause would be written: *)
+(* The above typically occurs in a multi-pattern match clause, so the clause
+   expression is on a line by itself.  This is the more typical way a long
+   single-pattern match clause would be written: *)
 let should_check_can_sell_and_marking regulatory_regime =
   match z with
   | `foo ->
@@ -118,13 +118,18 @@ let _ =
       <= b)
   then ()
 let _ =
+  (* Comparisons are different than conditionals; we don't regard them as
+     conceptually part of the [if] expression. *)
   if a
      <= b
   then ()
 let _ =
+  (* We regard the outermost condition terms as conceptually part of the [if]
+     expression and indent accordingly.  Whether [&&] or [||], conditionals
+     effectively state lists of conditions for [then]. *)
   if Edge_adjustment.is_zero arb.cfg.extra_edge
-     && 0. = sys.plugs.edge_backoff
-     && 0. = zero_acvol_edge_backoff
+  && 0. = sys.plugs.edge_backoff
+  && 0. = zero_acvol_edge_backoff
   then 0.
   else 1.
 let _ =
